# -*- coding:utf-8 -*-
# !/usr/bin/env python 3.9.11
"""
@file    :  factory.py
@time    :  2021/8/5 5:21 pm
@author  :  YuYanQing
@version :  1.0
@contact :  mryu168@163.com
@license :  (c)copyright 2022-2026
@desc    :  函数助手
"""
import json
import logging
import random
import re
import string
from typing import List
from urllib.parse import quote

import pypinyin
from faker import Faker
from requests.exceptions import InvalidURL
from urllib3.exceptions import LocationParseError
from urllib3.util import parse_url
from custard.core.processor import DataKitHelper

from custard.time.moment import Moment
from custard.utils.id_cards import IdNumber

fake = Faker(["zh_CN"])
logger = logging.getLogger(__name__)


class MockHelper:
    @staticmethod
    def rand_hans2pinyin(hans, style="A"):
        """
        汉字转拼音
        Args:
            hans: 汉字
            style: 返回首字母还是全拼, A:全拼; F:首字母
        Returns:
        Examples:
            >>> print(MockHelper.hans2pinyin("啊哈哈哈"))
            >>> print(MockHelper.hans2pinyin("啊哈哈哈", "F"))
        """

        if style.upper() == "F":
            return "".join(pypinyin.lazy_pinyin(hans=hans, style=pypinyin.Style.FIRST_LETTER))
        else:
            return "".join(pypinyin.lazy_pinyin(hans=hans))

    @staticmethod
    def rand_id_card(random_sex):
        """
        随机生成有效身份证
        Args:
            random_sex: 性别
        Returns:
        Examples:
            >>> MockHelper.rand_id_card()
        """
        return IdNumber.get_id_card(random_sex)

    @staticmethod
    def rand_mail(email_type=None, max_num=None, rad_count=None):
        """
        Args:
            email_type: 邮箱类型
            max_num: 邮箱地址最大长度
            rad_count: 所生成的数量
        Returns:
        Examples:
            >>> MockHelper.rand_mail(email_type="@qq.com", max_num=10, rad_count=5)
        """
        temp = []
        count = 0
        email_list = []
        email_array = [
            "@126.com",
            "@163.com",
            "@sina.com",
            "@sohu.com",
            "@yahoo.com.cn",
            "@gmail.com",
            "@yahoo.com",
        ]
        email_type_ = random.choice(email_array) if email_type is None else email_type
        max_num_ = random.randint(5, 10) if max_num is None else max_num
        rad_count_ = 1 if rad_count is None else rad_count
        while count < rad_count_:
            for _i in range(0, max_num_):
                status = random.randint(0, 1)
                if status == 0:
                    temp.append(random.choice(string.ascii_letters))
                else:
                    temp.append(str(random.randint(0, 1)))
            temp_str = "".join(temp)
            # 每次转化后就丢弃temp、避免出现遍历追加['vjt000ho@qq.com', 'vjt000ho0110fm0w@qq.com'............]
            temp.clear()
            email_list.append(temp_str + email_type_)
            count += 1
        return email_list

    @staticmethod
    def rand_verify_code(max_num: int, rad_count: int):
        """
        随机生成6位的验证码
        Args:
            max_num: 最多可生成的长度
            rad_count: 需要生成的数量
        Returns:
        Examples:
            >>> MockHelper.rand_verify_code(max_num=6, rad_count=1)
        """
        # 注意: 这里我们生成的是0-9a-za-z的列表,当然你也可以指定这个list,这里很灵活
        # 比如: code_list = ['p','y','t','h','o','n'] # python的字母
        count = 0
        verification_codes = []
        while count < rad_count:
            code_list = []
            for i in range(10):  # 0-9数字
                code_list.append(str(i))
            for i in range(65, 91):  # 对应从“a”到“z”的ascii码
                code_list.append(chr(i))
            for i in range(97, 123):  # 对应从“a”到“z”的ascii码
                code_list.append(chr(i))
            # 从list中随机获取6个元素,作为一个片断返回
            my_slice = random.sample(code_list, max_num)
            verification_codes.append("".join(my_slice))  # list to string
            count += 1
        if rad_count > 1:
            return verification_codes
        else:
            return verification_codes[0]

    @staticmethod
    def rand_str_list(length):
        """
        生成给定长度的字符串,返回列表格式
        Args:
            length:
        Returns:
        Examples:
            >>> MockHelper.rand_str_list(length=5)
        """
        init_chars = "".join("".join(map(str, [i for i in range(10) if i != 4])))  # 数字
        sample_list = random.sample(init_chars, length)
        return sample_list

    @staticmethod
    def rand_str(num_length):
        """
        从a-za-z0-9生成指定数量的随机字符
        Args:
            num_length:
        Returns:
        Examples:
            >>> MockHelper.rand_str(6)
        """
        return [random.choice(string.digits + string.ascii_letters) for i in range(num_length)]

    @staticmethod
    def rand_mum(num_length):
        """
        9生成指定数量的随机数字
        Args:
            num_length:
        Returns:
        """
        return "".join([random.choice(string.digits) for i in range(num_length)])

    @staticmethod
    def rand_int_number(min_value: int = 0, max_value: int = 9999, step: int = 1) -> int:
        """
        随机生成整数
        Args:
            min_value:
            max_value:
            step:
        Returns:
        Examples:
            >>> MockHelper.rand_int_number()
        """
        return random.randrange(min_value, max_value + 1, step)

    @staticmethod
    def rand_float_number(start_num=0, end_num=9, accuracy=1):
        """
        随机生成浮点数
        Args:
            start_num:
            end_num:
            accuracy:
        Returns:
        Examples:
            >>> MockHelper.rand_float_number(start_num=0, end_num=6, accuracy=3)
        """
        try:
            start_num = int(start_num)
            end_num = int(end_num)
            accuracy = int(accuracy)
        except ValueError:
            raise AssertionError("调用随机整数失败,范围参数或精度有误!\n小数范围精度")
        num = random.uniform(start_num, end_num) if start_num <= end_num else random.uniform(end_num, start_num)
        return round(num, accuracy)

    @staticmethod
    def rand_compute_time(
        days=0,
        seconds=0,
        microseconds=0,
        milliseconds=0,
        minutes=0,
        hours=0,
        weeks=0,
        custom=None,
    ):
        """
        随机生成偏移时间
        Args:
            days:
            seconds:
            microseconds:
            milliseconds:
            minutes:
            hours:
            weeks:
            custom:

        Returns:

        """
        return str(
            Moment.skew_date(
                days=days,
                seconds=seconds,
                microseconds=microseconds,
                milliseconds=milliseconds,
                minutes=minutes,
                hours=hours,
                weeks=weeks,
                custom=custom,
            ),
        )

    @staticmethod
    def rand_lowercase_letter(length) -> str:
        """
        Generate a random lowercase ASCII letter (a-z).
        Args:
            length:

        Returns:

        Examples:
            >>> MockHelper.rand_lowercase_letter(10)
        """

        return "".join([random.choice(string.ascii_lowercase) for i in range(length)])

    @staticmethod
    def rand_uppercase_letter(length) -> str:
        """
        Generate a random uppercase ASCII letter (A-Z).
        Args:
            length:

        Returns:

        Examples:
            >>> MockHelper.rand_uppercase_letter(10)
        """
        return "".join([random.choice(string.ascii_uppercase) for i in range(length)])

    @staticmethod
    def rand_sample(length=10):
        """
        随机生成字符(英文+数字
        Args:
            length:

        Returns:

        Examples:
            >>> MockHelper.rand_sample(10)

        """
        return "".join([random.choice(string.ascii_letters + string.digits) for i in range(length)])

    @staticmethod
    def rand_mobile_number(length: int):
        """
        随机生成手机号
        Returns:
        Examples:
            >>> MockHelper.rand_mobile_number(1)
        """
        phone_prefixes = [
            134,
            135,
            136,
            137,
            138,
            139,
            147,
            150,
            151,
            152,
            157,
            158,
            159,
            182,
            187,
            188,
            130,
            131,
            132,
            145,
            155,
            156,
            185,
            186,
            145,
            133,
            153,
            180,
            181,
            189,
        ]
        rand_phone_prefix = phone_prefixes[random.randint(0, len(phone_prefixes))]
        return [f"{rand_phone_prefix}{MockHelper.rand_mum(8)}" for i in range(length)]

    @staticmethod
    def rand_name():
        """
        随机生成名字
        Returns:
        Examples:
            >>> MockHelper.rand_name()
        """
        return fake.name()

    @staticmethod
    def rand_address():
        """
        随机生成所在地址
        Returns:
        Examples:
            >>> MockHelper.rand_address()
        """
        return fake.address()

    @staticmethod
    def rand_country():
        """
        随机生成国家名
        Returns:
        Examples:
            >>> MockHelper.rand_country()
        """
        return fake.country()

    @staticmethod
    def rand_country_code():
        """
        随机生成国家代码
        Returns:
        Examples:
            >>> MockHelper.rand_country_code()
        """
        return "".join(fake.country_code())

    @staticmethod
    def rand_city_name():
        """
        随机生成城市名
        Returns:
        Examples:
            >>> MockHelper.rand_city_name()
        """
        return fake.city_name()

    @staticmethod
    def rand_city():
        """
        随机生成城市
        Returns:
        Examples:
            >>> MockHelper.rand_city()
        """
        return fake.city()

    @staticmethod
    def rand_province():
        """
        随机生成省份
        Returns:
        Examples:
            >>> MockHelper.rand_province()
        """
        return fake.province()

    @staticmethod
    def rand_email():
        """
        随机生成email
        Returns:
        Examples:
            >>> MockHelper.rand_email()
        """
        return fake.email()

    @staticmethod
    def rand_ipv4():
        """
        随机生成ipv4地址
        Returns:
        Examples:
            >>> MockHelper.rand_ipv4()
        """
        return fake.ipv4()

    @staticmethod
    def rand_license_plate():
        """
        随机生成车牌号
        Returns:
        Examples:
            >>> MockHelper.rand_license_plate()
        """
        return fake.license_plate()

    @staticmethod
    def rand_color():
        """
        随机生成颜色
        Returns:
        Examples:
            >>> MockHelper.rand_color()
        """
        return fake.rgb_color()

    @staticmethod
    def rand_safe_hex_color():
        """
        随机生成16进制的颜色
        Returns:
        Examples:
            >>> MockHelper.rand_safe_hex_color()
        """
        return fake.safe_hex_color()

    @staticmethod
    def rand_color_name():
        """
        随机生成颜色名字
        Returns:
        Examples:
            >>> MockHelper.rand_color_name()
        """
        return fake.color_name()

    @staticmethod
    def rand_company_name():
        """
        随机生成公司名
        Returns:
        Examples:
            >>> MockHelper.rand_company_name()
        """
        return fake.company()

    @staticmethod
    def rand_job():
        """
        随机生成工作岗位
        Returns:
        Examples:
            >>> MockHelper.rand_job()
        """
        return fake.job()

    @staticmethod
    def rand_pwd(length=10, special_chars=True, digits=True, upper_case=True, lower_case=True):
        """
        随机生成密码
        lower_case:
        upper_case:
        digits:
        special_chars:
        length:
        Returns:
        """
        return fake.password(
            length=length,
            special_chars=special_chars,
            digits=digits,
            upper_case=upper_case,
            lower_case=lower_case,
        )

    @staticmethod
    def rand_uuid4():
        """
        随机生成uuid
        Returns:
        Examples:
            >>> MockHelper.rand_uuid4()
        """
        return fake.uuid4()

    @staticmethod
    def rand_sha1(raw_output=False):
        """
        随机生成sha1
        Returns:
        Examples:
            >>> MockHelper.rand_sha1()
        """
        return fake.sha1(raw_output=raw_output)

    @staticmethod
    def rand_md5(raw_output=False):
        """
        随机生成md5
        Args:
            raw_output:

        Returns:

        Examples:
            >>> MockHelper.rand_md5()
        """
        return fake.md5(raw_output=raw_output)

    @staticmethod
    def rand_female():
        """
        随机生成女性名字
        Returns:

        Examples:
            >>> MockHelper.rand_female()
        """
        return fake.name_female()

    @staticmethod
    def rand_male():
        """
        随机生成男性名字
        Returns:
        Examples:
            >>> MockHelper.rand_male()
        """
        return fake.name_male()

    @staticmethod
    def rand_user_info(sex=None):
        """
        随机生成粗略的基本信息
        Args:
            sex:

        Returns:

        Examples:
            >>> MockHelper.rand_user_info()
        """
        return fake.simple_profile(sex=sex)

    @staticmethod
    def rand_user_info_pro(fields=None, sex=None):
        """
        随机生成详细的基本信息
        Args:
            fields:
            sex:

        Returns:

        Examples:
            >>> MockHelper.rand_user_info_pro()
        """
        return fake.profile(fields=fields, sex=sex)

    @staticmethod
    def rand_user_agent():
        """
        随机生成浏览器头user_agent
        Returns:

        Examples:
            >>> MockHelper.rand_user_agent()
        """
        return fake.user_agent()

    @staticmethod
    def get_user_vars(target_key=None):
        """
        组合静态跟动态变量
        Args:
            target_key: 目标key

        Returns:

        Examples:
            >>> MockHelper.get_user_vars()
        """
        return target_key

    @staticmethod
    def get_encrypt_vars(target_key=None):
        """
        获取提取后的参数
        Args:
            target_key: 目标key
        Returns:
        Examples:
            >>> MockHelper.get_encrypt_vars()
        """
        return target_key

    @staticmethod
    def set_encrypt_vars(method, target_key):
        """
        给参数加密
        Args:
            method: 加密风格 base64 md5 sha1
            target_key:

        Returns:
        Examples:
            >>> MockHelper.set_encrypt_vars("base64_decode", "${randint_number}")
            >>> MockHelper.set_encrypt_vars("base64_decode", "$var_test_001")
            >>> MockHelper.set_encrypt_vars("base64_encode", "{{custom_null_var}}")
        """
        func_list = [
            "base64_encode",
            "base64_decode",
            "md5_encode",
            "sha1_decode",
            "jwt_decode",
            "jwt_encode",
        ]
        if method in func_list:
            exec('_var = {method}("{target_key}")'.format(method=method, target_key=MockHelper.cite(target_key)))
            var = locals()["_var"]
            return var.decode() if isinstance(var, bytes) else var
        else:
            raise ModuleNotFoundError("暂时仅支持:%s" % (", ".join(func_list)))

    @staticmethod
    def cite(name: str):
        """
        函数助手,输出以下常用随机数,返回结果值。支持的函数详情见func_dict:
        Args:
            name:  函数名,需要在func_dict存在的key值
        Returns:  随机函数调用结果 or None
        Examples:
            >>> MockHelper.cite('${rand_int_number()}')
            >>> MockHelper.cite('${rand_int_number(1,55)}')
            >>> MockHelper.cite('${rand_lowercase_letter(5)}')
            >>> MockHelper.cite('${rand_sample(1235)}')
            >>> MockHelper.cite("${get_user_vars()}")
            >>> MockHelper.cite("${get_user_vars(rand_pwd)}")
            >>> MockHelper.cite("${rand_user_agent()}")
            >>> MockHelper.cite("{{custom_none_var}}")
            >>> MockHelper.cite("{{custom_null_var}}")
            >>> MockHelper.cite("$var_test_001")
            >>> MockHelper.cite('$enc_(base64_encode,base64参数加密)')
        """
        # fix: file "d:\program files\python37\lib\re.py", line 173, in match
        # return _compile(pattern, flags).match(string)
        # typeerror: expected string or bytes-like object
        rand_vars = re.match("\\$\\{rand_(.*)\\((.*)\\)\\}", str(name))  # 带参数
        rand_no_vars = re.match("\\$\\{rand_(.*)\\}", str(name))  # 无参数
        dynamic_vars = re.match("\\$\\{get(.*)\\((.*)\\)\\}", str(name))  # 动态自定义
        own_vars = re.match("\\{\\{(.*)\\}\\}", str(name))  # 动态自定义
        extract_vars = re.match("\\$var_(.*)", str(name).upper())  # 后置提取参数
        lock_vars = re.match("\\$enc_(.*)", str(name))  # 带参数
        pattern = rand_vars if rand_vars is not None else dynamic_vars
        func_dict = {
            "mum": MockHelper.rand_mum,
            "hans2pinyin": MockHelper.rand_hans2pinyin,
            "id_card": MockHelper.rand_id_card,
            "mail": MockHelper.rand_mail,
            "int_number": MockHelper.rand_int_number,
            "float_number": MockHelper.rand_float_number,
            "compute_time": MockHelper.rand_compute_time,
            "lowercase_letter": MockHelper.rand_lowercase_letter,
            "uppercase_letter": MockHelper.rand_uppercase_letter,
            "sample": MockHelper.rand_sample,
            "mobile_number": MockHelper.rand_mobile_number,
            "name": MockHelper.rand_name,
            "address": MockHelper.rand_address,
            "country": MockHelper.rand_country,
            "country_code": MockHelper.rand_country_code,
            "city_name": MockHelper.rand_city_name,
            "city": MockHelper.rand_city,
            "province": MockHelper.rand_province,
            "email": MockHelper.rand_email,
            "ipv4": MockHelper.rand_ipv4,
            "license_plate": MockHelper.rand_license_plate,
            "color": MockHelper.rand_color,
            "rand_safe_hex_color": MockHelper.rand_safe_hex_color,
            "color_name": MockHelper.rand_color_name,
            "company_name": MockHelper.rand_company_name,
            "job": MockHelper.rand_job,
            "pwd": MockHelper.rand_pwd,
            "uuid4": MockHelper.rand_uuid4,
            "sha1": MockHelper.rand_sha1,
            "md5": MockHelper.rand_pwd,
            "female": MockHelper.rand_female,
            "male": MockHelper.rand_male,
            "user_info": MockHelper.rand_user_info,
            "user_info_pro": MockHelper.rand_user_info_pro,
            "user_agent": MockHelper.rand_user_agent,
            "user_vars": MockHelper.get_user_vars,
            "encrypt_vars": MockHelper.get_encrypt_vars,
        }
        if pattern is not None:
            key, value = pattern.groups()
            if func_dict.get(key):
                func = func_dict[key]
                _param = [eval(x) if x.strip().isdigit() else x for x in value.split(",")]
                if len(_param) >= 1 and "" not in _param:
                    return func.__call__(*_param)
                elif "" in _param:
                    return func.__call__()  # 没有带参数的
                return None
            return None
        elif own_vars:
            vars_ = MockHelper.get_user_vars(own_vars.group().strip("{}"))
            return MockHelper.cite(vars_)
        elif extract_vars:
            return MockHelper.get_encrypt_vars(extract_vars.group())
        elif rand_no_vars:
            return func_dict[rand_no_vars.group().strip("${rand_}")].__call__()
        elif lock_vars:
            _lock_param = [eval(x) if x.strip().isdigit() else x for x in lock_vars.group().strip("$enc_()").split(",")]
            if len(_lock_param) < 2:
                raise IndexError(_lock_param)
            else:
                return MockHelper.set_encrypt_vars.__call__(*_lock_param)
        else:
            return name  # 函数名不存在返回原始值

    @staticmethod
    def comb_data(dict_map: dict) -> dict:
        """
        合并参数化数据
        Args:
            dict_map: 初始data dict类型

        Returns: 转化后的数据 若无则返回原始值

        Examples:
            >>> MockHelper.comb_data({"product": {"brand_id": "{{int}}", "category_id": '${rand_float_number(1,2,3)}'}})
            >>> MockHelper.comb_data({"now_time": "${rand_compute_time()}"})
            >>> MockHelper.comb_data({"key1":"$enc_(base64,base64参数加密)"})
        """
        if isinstance(dict_map, dict):
            for key in list(dict_map.keys()):
                if isinstance(dict_map[key], list):
                    for i in range(len(dict_map[key])):
                        dict_map[key][i] = MockHelper.comb_data(dict_map=dict_map[key][i])
                elif isinstance(dict_map[key], dict):
                    dict_map[key] = MockHelper.comb_data(dict_map=dict_map[key])
                else:
                    dict_map[key] = MockHelper.cite(dict_map[key])
            return dict_map
        return None


class AutoVivification(dict):
    def __getitem__(self, item):
        try:
            return dict.__getitem__(self, item)
        except KeyError:
            value = self[item] = type(self)()
        return value


class MsHelper(DataKitHelper):
    GLOBAL_PAGE_INDEX = ["page_index", "pageindex"]
    GLOBAL_PAGE_SIZE = ["page_size", "pagesize"]

    @classmethod
    def __property__(cls, prop):
        """
        数据转换
        Args:
            prop:
        Returns:
        Examples:
            >>> props = [1,"abc",True,None,False]
            >>> for prop in props:
            ...    MsHelper.__property__(prop)
        """
        randint_val = random.randint(1, 1000)
        if isinstance(prop, bool):
            prop = "true" if prop else "false"
        if isinstance(prop, int):
            prop = randint_val
        elif isinstance(prop, float):
            prop = randint_val / 3.333
        elif prop is None:
            prop = "null"
        return prop

    @classmethod
    def obj_convert_dict(cls, data):
        """
        字典类型转换
        Args:
            data:
        Returns:
        """
        support_type = (list, float, int, tuple)
        type_err = TypeError("类型错误、仅支持dict")
        if isinstance(data, support_type):
            raise type_err
        if isinstance(data, str):
            try:
                return cls.safely_json_loads(data)
            except json.decoder.JSONDecodeError as decoder_err:
                raise Exception(decoder_err)
        if isinstance(data, dict):
            return data
        return None

    @classmethod
    def sub_kv(cls, data: List):
        """
        合并key value
        Args:
            data:
        Returns:
        Examples:
            >>> data_ = ["a1=5","a3=6"]
            >>> dint_data_ = ["a2=6","a2=6"]
            >>> MsHelper.sub_kv(data_)
            >>> MsHelper.sub_kv(dint_data_)
        """
        temp_dict = {}
        for index in range(len(data)):
            present = data[index]
            if "=" in present:
                pr_ = present.split("=")
                temp_dict.update({pr_[0]: pr_[1]})
            else:
                temp_dict.update({present: ""})
            # 相同数据跳出
            if index < len(data) - 1 and present == data[index + 1]:
                break
        return temp_dict

    @classmethod
    def url_to_json(cls, url):
        """
        form-data 转换为 Json
        Args:
            url:
        Returns:
        Examples:
            >>> form_data_ = ""
            >>> for index in range(10000):
            ...     form_data_ += f"&a{index}={index}"
            >>> url_ = f"http://localhost?{form_data_}"
            >>> output_ = MsHelper.url_to_json(url_)
            >>> print(output_)
        """
        try:
            scheme, auth, host, port, path, query, fragment = parse_url(url)
        except LocationParseError as e:
            raise InvalidURL(*e.args)
        output = {}
        if query:
            query_split = query.split("&")
            start = 0
            end = len(query_split) - 1
            while start <= end:
                spl_data = [query_split[start], query_split[end]]
                output = dict(output, **cls.sub_kv(spl_data))
                start += 1
                end -= 1
        return output

    @classmethod
    def json2form(cls, obj):
        """
        json转换为fromdata
        Args:
            obj:
        Returns:
        Examples:
            >>> have_array_obj_ = {"a":125678,"b":"ABCDEFG", "c":[{"c1":5}]}
            >>> have_dict_obj_ = {"a":125678,"b":"ABCDEFG", "c":{"c1":5}}
            >>> err_obj_ = {"a":125678,"b":"ABCDEFG", "c":[{"c1":5}],  "d":{"d1":5}}
            >>> obj_ = {"a":125678,"b":"%", "c":"=","d":"&", "e":""}
            >>> MsHelper.json2form(json.dumps(obj_))
            >>> MsHelper.json2form(json.dumps(have_array_obj_))
            >>> MsHelper.json2form(json.dumps(have_dict_obj_))
            >>> MsHelper.json2form(json.dumps(err_obj_))
        """
        output = ""
        data = cls.obj_convert_dict(obj)
        for key, value in data.items():
            if isinstance(value, (list, dict)):
                raise TypeError("请核对参数及Content-Type是否规范")
            if isinstance(value, str):
                value = quote(value, "unicode")
            output += f"&{key}={value}"
        return output

    @classmethod
    def json2vars(cls, target_data, source_data="", replace=True):
        """
        json转为vars
        Args:
            target_data:
            source_data:
            replace:
        Returns:
        Examples:
            >>> data_ = { "code": 200, "message": None, "error": False, "details":[{"d1":True}], "total_count": 5}
            >>> result = MsHelper.json2vars(data_)
        """
        if isinstance(target_data, dict):
            for key, value in target_data.items():
                if not isinstance(value, (list, dict)):
                    if value is None:
                        value = "null"
                    elif isinstance(value, bool):
                        value = "true" if value else "false"
                    if replace:
                        paging = [cls.GLOBAL_PAGE_INDEX, cls.GLOBAL_PAGE_SIZE]
                        if key in cls.GLOBAL_PAGE_INDEX:
                            value = 1
                        elif key in cls.GLOBAL_PAGE_SIZE:
                            value = 10
                        if key not in paging:
                            value = cls.__property__(value)
                    source_data += f'vars.put("{key}","{value}");\n'
                else:
                    source_data = cls.json2vars(target_data=value, source_data=source_data, replace=replace)
        elif isinstance(target_data, list):
            for index in range(len(target_data)):
                target_data_ = target_data[index]
                # 启用该行数据不会去重,有多少条就产生多少
                # source_data = cls.json2vars(target_data=target_data_, source_data=source_data, replace=replace)
                # 以下方式会去重
                return cls.json2vars(target_data=target_data_, source_data=source_data, replace=replace)
        return source_data



# -*- coding:utf-8 -*-
# !/usr/bin/env python 3.9.11
"""
@File    :  utils.py
@Time    :  2022/7/18 10:16
@Author  :  YuYanQing
@Version :  1.0
@Contact :  mryu168@163.com
@License :  (C)Copyright 2022-2026
@Desc    :  None
"""
import re
import string


def path_format(url, params):
    t = string.Template(re.subn(r"{(\w+)}", r"${\1}", url)[0])
    return t.safe_substitute(params)


def form_format(data: dict, file: str):
    _data = {}
    for k, v in data.items():
        v = (k, open(file, mode="rb")) if v == file else (None, v)
        _data.update({k: v})
    return _data



# -*- coding:utf-8 -*-
# !/usr/bin/env python 3.9.11
"""
@File    :  kerberos.py
@Time    :  2020/9/25 19:55
@Author  :  YuYanQing
@Version :  1.0
@Contact :  mryu168@163.com
@License :  (C)Copyright 2022-2026
@Desc    :  keys distributing center (密钥分配中心)
"""
import base64
import hashlib
from datetime import datetime, timedelta
from hashlib import sha1

import jwt


class Kerberos:
    @staticmethod
    def jwt_encode(secret_key, target_value, seconds):
        """
        jwt加密
        Args:
            secret_key:
            target_value:
            seconds:
        Returns:
        Examples:
            >>> JWT_SECRET_KEY = '1235678AC51Y55'
            >>> token = Kerberos.jwt_encode(secret_key=JWT_SECRET_KEY,
            ... target_value={"key1": "value1"}, seconds=0)
            >>> print(token)
            >>> import time
            >>> time.sleep(1)
            >>> print(Kerberos.jwt_decode(secret_key=JWT_SECRET_KEY, target_value=token))
        """
        return jwt.encode(
            {
                "target_value": target_value,
                "iat": datetime.utcnow() + timedelta(seconds=seconds),
                "exp": datetime.utcnow(),
            },
            secret_key,
            algorithm="HS256",
        )

    @staticmethod
    def jwt_decode(secret_key, target_value):
        """
        jwt 解密
        Args:
            secret_key:
            target_value:
        Returns:
        """
        return jwt.decode(target_value, secret_key, algorithms="HS256")

    @staticmethod
    def base64_encode(key: str):
        """
        base64 算法加密
        Args:
            key: 加密后的字符
        Returns:
        Examples:
            >>> print(Kerberos.base64_encode("Abc@1235678"))
        """
        return base64.b64encode(key.encode())

    @staticmethod
    def base64_decode(binary: str):
        """
        base64 算法解密
        Args:
            binary: 需要转成2进制格式才可以转换,所以我们这里再手动转换一下
        Returns:
        Examples:
            >>> print(Kerberos.base64_decode("QWJjQDEyMzU2Nzg="))
        """
        missing_padding = 4 - len(binary) % 4
        if missing_padding:
            binary += "=" * missing_padding
        return base64.b64decode(binary)

    @staticmethod
    def md5_encode(decode_msg: str):
        """
        md5 算法加密
        Args:
            decode_msg: 需加密的字符串
        Returns:
        Examples:
            >>> print(Kerberos.md5_encode("1235678"))
        """
        return hashlib.md5(str(decode_msg).encode(encoding="utf-8")).hexdigest()

    @staticmethod
    def sha1_decode(decode_msg: str):
        """
        哈希算法加密
        Args:
            decode_msg: 需加密的字符串
        Returns:
        Examples:
            >>> print(Kerberos.sha1_decode("Hello Word sha1Encrypt"))
        """
        return sha1(decode_msg.encode("utf-8")).hexdigest()



# -*- coding:utf-8 -*-
# !/usr/bin/env python 3.9.11
"""
@File    :  async_sqlalchemy.py
@Time    :  2022/5/1 8:21 PM
@Author  :  YuYanQing
@Version :  1.0
@Contact :  mryu168@163.com
@License :  (C)Copyright 2022-2026
@Desc    :  None
"""
from __future__ import annotations

from typing import TYPE_CHECKING, Optional

from sqlalchemy import func, select

from .api import create_page, resolve_params
from .bases import AbstractPage, AbstractParams
from .sync_sqlalchemy import paginate_query

if TYPE_CHECKING:
    from sqlalchemy.ext.asyncio import AsyncSession
    from sqlalchemy.sql import Select


async def paginate(
    session: AsyncSession,
    query: Select,
    params: Optional[AbstractParams] = None,
) -> AbstractPage:  # pragma: no cover # FIXME: fix coverage report generation
    params = resolve_params(params)

    total = await session.scalar(select(func.count()).select_from(query.subquery()))  # type: ignore
    items = await session.execute(paginate_query(query, params))

    return create_page(items.scalars().unique().all(), total, params)


__all__ = ["paginate"]



# -*- coding:utf-8 -*-
# !/usr/bin/env python 3.9.11
"""
@File    :  dafunc.py
@Time    :  2022/5/15 10:37 PM
@Author  :  YuYanQing
@Version :  1.0
@Contact :  mryu168@163.com
@License :  (C)Copyright 2022-2026
@Desc    :  None
"""

import copy
import sys
import time
import types
from functools import wraps
from typing import Any

from .exceptions import FunctionTimedOut
from .stoppable_thread import StoppableThread

__all__ = ("func_timeout", "bind_timeout", "calc_time", "FunctionTimedOut", "TimerContextManager", "StoppableThread")


def func_timeout(timeout, func, args=(), kwargs=None):
    """
    Args:
        timeout:
        func:
        args:
        kwargs:
    Returns:
    """
    if not kwargs:
        kwargs = {}
    if not args:
        args = ()

    ret = []
    exception = []
    is_stopped = False

    def funcwrap(args2, kwargs2):
        try:
            ret.append(func(*args2, **kwargs2))
        except FunctionTimedOut:
            pass
        except Exception as e:
            exc_info = sys.exc_info()
            if is_stopped is False:
                e.__traceback__ = exc_info[2].tb_next
                exception.append(e)

    thread = StoppableThread(target=funcwrap, args=(args, kwargs))
    thread.daemon = True

    thread.start()
    thread.join(timeout)

    if thread.is_alive():
        is_stopped = True

        class FunctionTimedOutTempType(FunctionTimedOut):
            def __init__(self):
                return FunctionTimedOut.__init__(self, "", timeout, func, args, kwargs)

        stop_exception = type(
            "FunctionTimedOut" + str(hash("%d_%d_%d_%d" % (id(timeout), id(func), id(args), id(kwargs)))),
            FunctionTimedOutTempType.__bases__,
            dict(FunctionTimedOutTempType.__dict__),
        )
        thread.stop_thread(stop_exception)
        thread.join(min(0.1, timeout / 50.0))
        raise FunctionTimedOut("", timeout, func, args, kwargs)
    else:
        thread.join(0.5)

    if exception:
        raise exception[0] from None

    if ret:
        return ret[0]
    return None


def bind_timeout(timeout, allow_override=False):
    """
    Args:
        timeout:
        allow_override:
    Returns:
    """
    default_timeout = copy.copy(timeout)

    is_timeout_function = bool(
        issubclass(
            timeout.__class__,
            (
                types.FunctionType,
                types.MethodType,
                types.LambdaType,
                types.BuiltinFunctionType,
                types.BuiltinMethodType,
            ),
        ),
    )

    if not is_timeout_function and not issubclass(timeout.__class__, (float, int)):
        try:
            timeout = float(timeout)
        except ValueError:
            raise ValueError(
                " Passed type: < %s > is not of any of these, and cannot be converted to a float."
                % (timeout.__class__.__name__,),
            )

    if not allow_override and not is_timeout_function:

        def _function_decorator(func):
            return wraps(func)(lambda *args, **kwargs: func_timeout(default_timeout, func, args=args, kwargs=kwargs))

        return _function_decorator

    if not is_timeout_function:

        def _function_decorator(func):
            def _function_wrapper(*args, **kwargs):
                use_timeout = kwargs.pop("force_timeout") if "force_timeout" in kwargs else default_timeout

                return func_timeout(use_timeout, func, args=args, kwargs=kwargs)

            return wraps(func)(_function_wrapper)

        return _function_decorator

    timeout_function = timeout

    if allow_override:

        def _function_decorator(func):
            def _function_wrapper(*args, **kwargs):
                if "force_timeout" in kwargs:
                    use_timeout = kwargs.pop("force_timeout")
                else:
                    use_timeout = timeout_function(*args, **kwargs)

                return func_timeout(use_timeout, func, args=args, kwargs=kwargs)

            return wraps(func)(_function_wrapper)

        return _function_decorator

    def _function_decorator(func):
        def _function_wrapper(*args, **kwargs):
            use_timeout = timeout_function(*args, **kwargs)

            return func_timeout(use_timeout, func, args=args, kwargs=kwargs)

        return wraps(func)(_function_wrapper)

    return _function_decorator


class TimerContextManager(object):
    """
    用上下文管理器计时,可对代码片段计时
    """

    def __init__(self, is_print_log=True, logger=None):
        self._is_print_log = is_print_log
        self.logger = logger
        self.t_spend = None
        self._line = None
        self._file_name = None
        self.time_start = None

    def __enter__(self):
        self._line = sys._getframe().f_back.f_lineno  # 调用此方法的代码的函数
        self._file_name = sys._getframe(1).f_code.co_filename  # 哪个文件调了用此方法
        self.time_start = time.time()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.t_spend = time.time() - self.time_start
        if self._is_print_log:
            msg = f'对下面代码片段进行计时:  \n执行"{self._file_name}:{self._line}" 用时 {round(self.t_spend, 2)} 秒'
            if self.logger:
                self.logger.info(msg)
            else:
                print(msg)


def calc_time(logger: Any = None):
    """
    统计给目标函数运行时间
    Args:
        logger:
    Returns:
    Examples:
        >>> @calc_time("(自定义模块名)")
        ... def test_calc_time(num=100):
        ...     return sum([x for x in range(num + 1)])
        >>> test_calc_time(5)
    """

    def _calc_time(func):
        @wraps(func)
        def __calc_time(*args, **kwargs):
            start_time = time.time()
            # 定义result接收函数返回值,并且在装饰函数最后返回回去
            result = func(*args, **kwargs)
            elapsed_time = time.time() - start_time
            if logger:
                logger.info(f"{func.__name__} cost {elapsed_time:.4f} seconds")
            else:
                print(f"{func.__name__} cost {elapsed_time:.4f} seconds")
            return result

        return __calc_time

    return _calc_time



# -*- coding:utf-8 -*-
# !/usr/bin/env python 3.9.11
"""
@File    :  crypto.py
@Time    :  2020/9/25 19:55
@Author  :  YuYanQing
@Version :  1.0
@Contact :  mryu168@163.com
@License :  (C)Copyright 2022-2026
@Desc    :  None
"""

import base64
import copy
import logging
import os
import struct
from abc import abstractmethod

from Crypto import Random
from Crypto.Cipher import AES, PKCS1_OAEP, PKCS1_v1_5
from Crypto.PublicKey import RSA
from Crypto.Util import Counter
from six import text_type

from .streambody import StreamBody

logger = logging.getLogger(__name__)

_AES_CTR_COUNTER_BITS_LENGTH = 8 * 16
_AES_256_KEY_SIZE = 32
__all__ = ["BaseProvider", "AESProvider", "RSAProvider", "AESCTRCipher", "DataDecryptAdapter", "DataEncryptAdapter"]


def to_bytes(str_):
    """
    将字符串转为bytes
    Args:
        str_:

    Returns:

    """
    if isinstance(str_, text_type):
        try:
            return str_.encode("utf-8")
        except UnicodeEncodeError:
            detail = "your unicode strings can not encoded in utf8, utf8 support only!"
            raise CryptoException(detail)
    return str_


def random_key(key_len):
    """

    Args:
        key_len:

    Returns:

    """
    return Random.new().read(key_len)


def random_iv():
    """

    Returns:

    """
    iv = Random.new().read(16)
    return iv


def iv_to_big_int(iv):
    """

    Args:
        iv:

    Returns:

    """
    iv_pair = struct.unpack(">QQ", iv)
    iv_int = iv_pair[0] << 64 | iv_pair[1]
    return iv_int


class CryptoException(Exception):
    def __init__(self, message):
        self._message = message

    def __str__(self):
        return str(self._message)


class AESCTRCipher(object):
    """数据加密类,用于加密用户数据"""

    def __init__(self):
        """初始化"""
        self.__block_size_len = AES.block_size
        self.__cipher = None
        self.__key_len = _AES_256_KEY_SIZE

    def new_cipher(self, key, start, offset=0):
        """
        初始化AES加解密对象
        Args:
            key: 对称密钥
            start: 对称加密初始随机值
            offset: 主要用于解密,想要解密文本的偏移,必须为16的整数倍

        Returns:
        """
        block_index_offset = self.__calc_offset(offset)
        new_start = start + block_index_offset
        my_counter = Counter.new(_AES_CTR_COUNTER_BITS_LENGTH, initial_value=new_start)
        self.__cipher = AES.new(key, AES.MODE_CTR, counter=my_counter)

    def encrypt(self, plaintext):
        """
        需要加密的数据
        Args:
            plaintext:

        Returns:

        """
        if self.__cipher is None:
            raise CryptoException("cipher is not initialized")
        return self.__cipher.encrypt(plaintext)

    def decrypt(self, plaintext):
        """
        需要解密的数据,数据的起始位置必须为16的整数倍
        Args:
            plaintext:

        Returns:

        """
        if self.__cipher is None:
            raise CryptoException("cipher is not initialized")
        return self.__cipher.decrypt(plaintext)

    # offset 必须为block_size的整数倍
    def __calc_offset(self, offset):
        """
        通过文本偏移计算counter的偏移
        Args:
            offset:

        Returns:

        """
        if not self.__is_block_aligned(offset):
            raise CryptoException("offset is not align to encrypt block")
        return offset // self.__block_size_len

    def __is_block_aligned(self, offset):
        """
        判断文本偏移是否是block_size对齐
        Args:
            offset:

        Returns:

        """
        if offset is None:
            offset = 0
        return offset % self.__block_size_len == 0

    def adjust_read_offset(self, offset):
        """
        用于调整读取的offset为block_size对齐
        Args:
            offset:

        Returns:

        """

        if offset:
            offset = (offset // self.__block_size_len) * self.__block_size_len
        return offset

    def get_key(self):
        """获取随机密钥"""
        return random_key(self.__key_len)

    def get_counter_iv(self):
        """获取对称加密初始随机值"""
        return random_iv()


class RSAKeyPair:
    """封装有公钥和私钥"""

    def __init__(self, public_key, private_key):
        self.public_key = public_key
        self.private_key = private_key


class RSAKeyPairPath:
    """封装有公钥和私钥的路径"""

    def __init__(self, public_key_path, private_key_path):
        self.public_key_path = public_key_path
        self.private_key_path = private_key_path


class BaseProvider(object):
    """客户端主密钥加密基类"""

    def __init__(self, cipher):
        """初始化
        :param cipher(an AES object): 数据加解密类
        """
        self.data_cipher = cipher

    def get_data_key(self):
        """随机获取数据加解密密钥"""
        return self.data_cipher.get_key()

    @abstractmethod
    def init_data_cipher(self):
        """初始化cipher"""

    @abstractmethod
    def init_data_cipher_by_user(self, encrypt_key, encrypt_iv, offset=0):
        """根据密钥初始化cipher"""

    def adjust_read_offset(self, start):
        """用于调整读取的offset为block_size对齐"""
        return self.data_cipher.adjust_read_offset(start)

    def make_data_encrypt_adapter(self, stream):
        """创建数据流加密适配器"""
        size = 0
        if hasattr(stream, "__len__"):
            size = len(stream)
        elif hasattr(stream, "tell") and hasattr(stream, "seek"):
            current = stream.tell()
            stream.seek(0, os.SEEK_END)
            size = stream.tell()
            stream.seek(current, os.SEEK_SET)
        else:
            return None
        return DataEncryptAdapter(stream, size, copy.copy(self.data_cipher))

    def make_data_decrypt_adapter(self, rt, offset):
        """创建数据流解密适配器"""
        return DataDecryptAdapter(rt, copy.copy(self.data_cipher), offset)


class RSAProvider(BaseProvider):
    """客户端非对称主密钥加密类"""

    def __init__(self, key_pair_info=None, cipher=AESCTRCipher(), passphrase=None):
        """初始化"""
        super(RSAProvider, self).__init__(cipher=cipher)

        default_rsa_dir = os.path.expanduser("~/.local_rsa")
        default_public_key_path = os.path.join(default_rsa_dir, ".public_key.pem")
        default_private_key_path = os.path.join(default_rsa_dir, ".private_key.pem")
        self.__encrypt_obj = None
        self.__decrypt_obj = None
        self.__data_key = None
        self.__data_iv = None

        if isinstance(key_pair_info, RSAKeyPair):
            self.__encrypt_obj = PKCS1_v1_5.new(RSA.importKey(key_pair_info.public_key, passphrase=passphrase))
            self.__decrypt_obj = PKCS1_v1_5.new(RSA.importKey(key_pair_info.private_key, passphrase=passphrase))
        elif isinstance(key_pair_info, RSAKeyPairPath):
            self.__encrypt_obj, self.__decrypt_obj = self.__get_key_by_path(
                key_pair_info.public_key_path,
                key_pair_info.private_key_path,
                passphrase,
            )
        else:
            logger.info("key_pair_info is None, try to get key from default path")
            if os.path.exists(default_private_key_path) and os.path.exists(default_public_key_path):
                self.__encrypt_obj, self.__decrypt_obj = self.__get_key_by_path(
                    default_public_key_path,
                    default_private_key_path,
                    passphrase,
                )

        # 为用户自动创建rsa
        if self.__encrypt_obj is None and self.__decrypt_obj is None:
            logger.warning("fail to get rsa key, will generate key")
            private_key = RSA.generate(2048)
            public_key = private_key.publickey()

            self.__encrypt_obj = PKCS1_OAEP.new(public_key)
            self.__decrypt_obj = PKCS1_OAEP.new(private_key)

            if not os.path.exists(default_rsa_dir):
                os.makedirs(default_rsa_dir)

            with open(default_private_key_path, "wb") as f:
                f.write(private_key.exportKey(passphrase=passphrase))

            with open(default_public_key_path, "wb") as f:
                f.write(public_key.exportKey(passphrase=passphrase))

    @staticmethod
    def get_rsa_key_pair(public_key, private_key):
        """开放给用户,用于生成RSAKeyPair"""
        if public_key is None or private_key is None:
            raise CryptoException("public_key or private_key is not allowed to be None !!!")
        return RSAKeyPair(public_key, private_key)

    @staticmethod
    def get_rsa_key_pair_path(public_key_path, private_key_path):
        """开放给用户,用于生成RSAKeyPairPath"""
        if public_key_path is None or private_key_path is None:
            raise CryptoException("public_key or private_key is not allowed to be None !!!")
        return RSAKeyPairPath(public_key_path, private_key_path)

    def __get_key_by_path(self, public_path=None, private_path=None, passphrase=None):
        """用于从提供的路径中获取公钥和私钥"""
        if public_path is None or private_path is None:
            return None, None

        encrypt_obj, decrypt_obj = None, None

        if os.path.exists(public_path) and os.path.exists(private_path):
            with open(public_path, "rb") as f:
                encrypt_obj = PKCS1_OAEP.new(RSA.importKey(f.read(), passphrase=passphrase))

            with open(private_path, "rb") as f:
                decrypt_obj = PKCS1_OAEP.new(RSA.importKey(f.read(), passphrase=passphrase))

        return encrypt_obj, decrypt_obj

    def init_data_cipher(self):
        """初始化cipher"""
        self.__data_key = self.get_data_key()
        self.__data_iv = self.data_cipher.get_counter_iv()
        start = iv_to_big_int(self.__data_iv)
        self.data_cipher.new_cipher(self.__data_key, start)
        encrypt_key = self.__encrypt_obj.encrypt(self.__data_key)
        encrypt_iv = self.__encrypt_obj.encrypt(self.__data_iv)
        return encrypt_key, encrypt_iv

    def init_data_cipher_by_user(self, encrypt_key, encrypt_iv, offset=0):
        """根据密钥初始化cipher"""
        self.__data_key = self.__decrypt_obj.decrypt(encrypt_key)
        self.__data_iv = self.__decrypt_obj.decrypt(encrypt_iv)
        start = iv_to_big_int(self.__data_iv)
        self.data_cipher.new_cipher(self.__data_key, start, offset)


class AESProvider(BaseProvider):
    """客户端对称主密钥加密类"""

    def __init__(self, aes_key=None, aes_key_path=None, cipher=AESCTRCipher()):
        """初始化"""
        super(AESProvider, self).__init__(cipher=cipher)
        self.__ed_obj = None
        self.__data_key = None
        self.__data_iv = None
        self.__my_counter = Counter.new(_AES_CTR_COUNTER_BITS_LENGTH, initial_value=0)
        self.__aes_key = aes_key
        self.__aes_key_path = aes_key_path

    def init_ed_obj(self):
        default_aes_dir = os.path.expanduser("~/.local_aes")
        default_key_path = os.path.join(default_aes_dir, ".aes_key.pem")
        if self.__aes_key:
            aes_key = to_bytes(base64.b64decode(to_bytes(self.__aes_key)))
            self.__ed_obj = AES.new(aes_key, AES.MODE_CTR, counter=self.__my_counter)
        elif self.__aes_key_path:
            if os.path.exists(self.__aes_key_path):
                with open(self.__aes_key_path, "rb") as f:
                    aes_key = f.read()
                    aes_key = to_bytes(base64.b64decode(to_bytes(aes_key)))
                    self.__ed_obj = AES.new(aes_key, AES.MODE_CTR, counter=self.__my_counter)
        else:
            logger.info("aes_key and aes_key_path is None, try to get key from default path")
            if os.path.exists(default_key_path):
                with open(default_key_path, "rb") as f:
                    aes_key = f.read()
                    aes_key = to_bytes(base64.b64decode(to_bytes(aes_key)))
                    self.__ed_obj = AES.new(aes_key, AES.MODE_CTR, counter=self.__my_counter)

        if self.__ed_obj is None:
            logger.warning("fail to get aes key, will generate key")
            aes_key = random_key(_AES_256_KEY_SIZE)
            self.__ed_obj = AES.new(aes_key, AES.MODE_CTR, counter=self.__my_counter)
            if not os.path.exists(default_aes_dir):
                os.makedirs(default_aes_dir)

            with open(default_key_path, "wb") as f:
                aes_key = to_bytes(base64.b64encode(to_bytes(aes_key)))
                f.write(aes_key)

    def init_data_cipher(self):
        """初始化cipher"""
        self.__data_key = self.get_data_key()
        self.__data_iv = self.data_cipher.get_counter_iv()
        start = iv_to_big_int(self.__data_iv)
        self.data_cipher.new_cipher(self.__data_key, start)

        self.init_ed_obj()
        encrypt_key = self.__ed_obj.encrypt(self.__data_key)
        encrypt_iv = self.__ed_obj.encrypt(self.__data_iv)
        return encrypt_key, encrypt_iv

    def init_data_cipher_by_user(self, encrypt_key, encrypt_iv, offset=0):
        """根据密钥初始化cipher"""
        self.init_ed_obj()
        self.__data_key = self.__ed_obj.decrypt(encrypt_key)
        self.__data_iv = self.__ed_obj.decrypt(encrypt_iv)
        start = iv_to_big_int(self.__data_iv)
        self.data_cipher.new_cipher(self.__data_key, start, offset)


class DataEncryptAdapter(object):
    """用于读取经过加密后的的数据"""

    def __init__(self, data, content_len, data_cipher):
        self._data = to_bytes(data)
        self._data_cipher = data_cipher
        self._content_len = content_len
        self._read_len = 0

    @property
    def len(self):
        return self._content_len

    def read(self, length):
        """读取加密后的数据"""
        if self._read_len >= self._content_len:
            return ""

        if length is None or length < 0:
            bytes_to_read = self._content_len - self._read_len
        else:
            bytes_to_read = min(length, self._content_len - self._read_len)

        if isinstance(self._data, bytes):
            content = self._data[self._read_len : self._read_len + bytes_to_read]
        else:
            content = self._data.read(bytes_to_read)

        self._read_len += bytes_to_read
        content = self._data_cipher.encrypt(content)
        return content


class DataDecryptAdapter(StreamBody):
    """用于读取经过解密后的数据"""

    def __init__(self, rt, data_cipher, offset=0):
        """初始化
        :param rt(request object): request请求返回的对象
        :param data_cipher(an AES object): 数据加解密类
        :param offset(int): 数据读取的偏移量
        """
        super(DataDecryptAdapter, self).__init__(rt)
        self._data_cipher = data_cipher
        self._offset = offset

    def read(self, length, auto_decompress=False):
        """读取解密后的数据"""
        if self._read_len >= self._content_len:
            return ""

        if self._use_encoding and not auto_decompress:
            content = self._rt.raw.read(length)
        else:
            try:
                content = next(self._rt.iter_content(length))
            except StopIteration:
                return ""

        content = self._data_cipher.decrypt(content)
        if self._read_len < self._offset < self._read_len + len(content):
            content = content[self._offset :]
            self._read_len = self._offset
        return content



# -*- coding:utf-8 -*-
# !/usr/bin/env python 3.9.11
"""
@File    :  __init__.py
@Time    :  2022/5/27 12:09 AM
@Author  :  YuYanQing
@Version :  1.0
@Contact :  mryu168@163.com
@License :  (C)Copyright 2022-2026
@Desc    :  None
"""
from six import text_type

from .decode import unidecode
from .factory import MockHelper
from .functools import (
    BatchTask,
    ExcContextManager,
    bind_run_many_times,
    handle_exception,
    keep_circulating,
    where_is_it_called,
    sync_redis_lock,
    synchronized_lock,
    singleton_lock,
)
from .kerberos import Kerberos
from .processor import DataKitHelper
from .regular import RegEx
from .system import SystemHand
from .useragent import firefox, internet_explorer, opera, safari
from .xprint import xprint

__all__ = [
    "text_type",
    "unidecode",
    "MockHelper",
    "Kerberos",
    "DataKitHelper",
    "RegEx",
    "SystemHand",
    "firefox",
    "safari",
    "internet_explorer",
    "opera",
    "xprint",
    "ExcContextManager",
    "BatchTask",
    "handle_exception",
    "keep_circulating",
    "where_is_it_called",
    "bind_run_many_times",
    "singleton_lock",
    "sync_redis_lock",
    "synchronized_lock",
]
